@@ -942,12 +942,12 @@ int start, end; register unsigned char *text;
                                                 if (!DELIMITER) {
 							curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 							if (*curtextbegin == '\n') curtextbegin ++;
-							curtextend = text+1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+							curtextend = curtextbegin /*text-m1*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 							if (*curtextend == '\n') curtextend ++;
                                                 }
                                                 else {
                                                         curtextbegin = backward_delimiter(text, textbegin, D_pattern, D_length, OUTTAIL);
-                                                        curtextend = forward_delimiter(text+1, textend, D_pattern, D_length, OUTTAIL);
+                                                        curtextend = forward_delimiter(curtextbegin /*text-m1*/, textend, D_pattern, D_length, OUTTAIL);
                                                 }
 						if (!OUTTAIL || INVERSE) textbegin = curtextend;
 						else if (DELIMITER) textbegin = curtextend - D_length;
@@ -1190,12 +1190,12 @@ register unsigned char *text;
 						if (!DELIMITER) {
 							curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 							if (*curtextbegin == '\n') curtextbegin ++;
-							curtextend = text+1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+							curtextend = curtextbegin /*text-m1*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 							if (*curtextend == '\n') curtextend ++;
 						}
 						else {
 							curtextbegin = backward_delimiter(text, textbegin, tc_D_pattern, tc_D_length, OUTTAIL);
-							curtextend = forward_delimiter(text+1, textend, tc_D_pattern, tc_D_length, OUTTAIL);
+							curtextend = forward_delimiter(curtextbegin /*text-m1*/, textend, tc_D_pattern, tc_D_length, OUTTAIL);
 						}
 					}
 					/* else prev curtextbegin is OK: if full AND isn't found, DOWITHMASK is 0-ed so that we search at most 1 line below */
@@ -1385,6 +1385,7 @@ int
 m_short(text, start, end)
 int start, end; register uchar *text;
 {
+	int m1=1;
 	int PRINTED = 0;
 	int pat_index;
         unsigned char *oldtext;
@@ -1438,12 +1439,12 @@ int start, end; register uchar *text;
                                         if (!DELIMITER) {
 						curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 						if (*curtextbegin == '\n') curtextbegin ++;
-						curtextend = text+1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+						curtextend = curtextbegin /*text-m1*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 						if (*curtextend == '\n') curtextend ++;
                                         }
                                         else {
                                                 curtextbegin = backward_delimiter(text, textbegin, D_pattern, D_length, OUTTAIL);
-                                                curtextend = forward_delimiter(text+1, textend, D_pattern, D_length, OUTTAIL);
+                                                curtextend = forward_delimiter(curtextbegin /*text-m1*/, textend, D_pattern, D_length, OUTTAIL);
                                         }
 					if (!OUTTAIL || INVERSE) textbegin = curtextend;
 					else if (DELIMITER) textbegin = curtextend - D_length;
@@ -1602,6 +1603,7 @@ int
 tc_m_short(text, start, end)
 int start, end; register uchar *text;
 {
+	int m1=1;
 	int PRINTED = 0;
 	int pat_index;
         unsigned char *oldtext;
@@ -1650,12 +1652,12 @@ int start, end; register uchar *text;
 					if (!DELIMITER) {
 						curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 						if (*curtextbegin == '\n') curtextbegin ++;
-						curtextend = text+1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+						curtextend = curtextbegin /*text-m1*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 						if (*curtextend == '\n') curtextend ++;
 					}
 					else {
 						curtextbegin = backward_delimiter(text, textbegin, tc_D_pattern, tc_D_length, OUTTAIL);
-						curtextend = forward_delimiter(text+1, textend, tc_D_pattern, tc_D_length, OUTTAIL);
+						curtextend = forward_delimiter(curtextbegin /*text-m1*/, textend, tc_D_pattern, tc_D_length, OUTTAIL);
 					}
 				}
 				/* else prev curtextbegin is OK: if full AND isn't found, DOWITHMASK is 0-ed so that we search at most 1 line below */
