@@ -687,9 +687,15 @@ int m, oldm;
 			if(text > textend) return 0;
 			if(WORDBOUND) {
 				/* if(isalnum(*(unsigned char *)(text+1))) goto CONT; --> fixed by SHIOZAKI Takehiko <takehi-s@ascii.co.jp> */
-				if((text+1 <= textend) && isalnum(*(unsigned char *)(text+1)) && isalnum(*(unsigned char *)text)) goto CONT;	/* as if there was no match */
+				if((text+1 <= textend) && isalnum(*(unsigned char *)(text+1)) && isalnum(*(unsigned char *)text)) {
+					shift = 1;	/* bg 4/27/97 */
+					goto WCONT;	/* as if there was no match */
+				}
 				/* if(isalnum(*(unsigned char *)(text-m))) goto CONT; --> fixed by SHIOZAKI Takehiko <takehi-s@ascii.co.jp> */
-				if((textbegin <= (text-m)) && isalnum(*(unsigned char *)(text-m)) && isalnum(*(unsigned char *)(text-m+1))) goto CONT;	/* as if there was no match */
+				if((textbegin <= (text-m)) && isalnum(*(unsigned char *)(text-m)) && isalnum(*(unsigned char *)(text-m+1))) {
+					shift = 1;	/* bg 4/27/97 */
+					goto WCONT;	/* as if there was no match */
+				}
 				/* changed by Udi 11/7/94 to avoid having to set TR[] to W_delim */
 			}
 
@@ -698,12 +704,12 @@ int m, oldm;
 				if (!DELIMITER) {
 					curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 					if (*curtextbegin == '\n') curtextbegin ++;
-					curtextend = text+1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+					curtextend = curtextbegin; /*text-m*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 					if (*curtextend == '\n') curtextend ++;
 				}
 				else {
 					curtextbegin = backward_delimiter(text, textbegin, tc_D_pattern, tc_D_length, OUTTAIL);
-					curtextend = forward_delimiter(text+1, textend, tc_D_pattern, tc_D_length, OUTTAIL);
+					curtextend = forward_delimiter(curtextbegin/*text-m*/, textend, tc_D_pattern, tc_D_length, OUTTAIL);
 				}
 			}
 			else {
@@ -711,12 +717,12 @@ int m, oldm;
 				if (!DELIMITER) {
 					curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 					if (*curtextbegin == '\n') curtextbegin ++;
-					curtextend = text+1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+					curtextend = curtextbegin /*text-m*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 					if (*curtextend == '\n') curtextend ++;
 				}
 				else {
 					curtextbegin = backward_delimiter(text, textbegin, D_pattern, D_length, OUTTAIL);
-					curtextend = forward_delimiter(text+1, textend, D_pattern, D_length, OUTTAIL);
+					curtextend = forward_delimiter(curtextbegin/*text-m*/, textend, D_pattern, D_length, OUTTAIL);
 				}
 			}
 
@@ -928,6 +934,7 @@ CONT:
 			if (m == 1) shift = 0; else shift = 1;	/* ZZZZZZZZZZZZZZZZ check it out later */
 		}
 		else shift = d1;
+WCONT:	;
 	}
 
 	if (!SILENT && INVERSE && !COUNT && (lastout <= textend)) {
@@ -1236,24 +1243,24 @@ CHARTYPE **lastout;	/* in, out */
 		if (!DELIMITER) {
 			curtextbegin = text + *i; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 			if (*curtextbegin == '\n') curtextbegin ++;
-			curtextend = text + *i /* + 1 agrep() has i++ */; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+			curtextend = curtextbegin /*text -m + *i*/ /* + 1 agrep() has i++ */; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 			if (*curtextend == '\n') curtextend ++;
 		}
 		else {
 			curtextbegin = backward_delimiter(text + *i, text, tc_D_pattern, tc_D_length, OUTTAIL);
-			curtextend = forward_delimiter(text + *i /* + 1 agrep() has i++ */, textend, tc_D_pattern, tc_D_length, OUTTAIL);
+			curtextend = forward_delimiter(curtextbegin /*text -m + *i*/ /* + 1 agrep() has i++ */, textend, tc_D_pattern, tc_D_length, OUTTAIL);
 		}
 	}
 	else {
 		if (!DELIMITER) {
 			curtextbegin = text + *i; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 			if (*curtextbegin == '\n') curtextbegin ++;
-			curtextend = text + *i /* + 1 agrep() has i++ */; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+			curtextend = curtextbegin /*text -m + *i*/ /* + 1 agrep() has i++ */; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 			if (*curtextend == '\n') curtextend ++;
 		}
 		else {
 			curtextbegin = backward_delimiter(text + *i, text, D_pattern, D_length, OUTTAIL);
-			curtextend = forward_delimiter(text + *i /* + 1 agrep() has i++ */, textend, D_pattern, D_length, OUTTAIL);
+			curtextend = forward_delimiter(curtextbegin /*text -m + *i*/ /* + 1 agrep() has i++ */, textend, D_pattern, D_length, OUTTAIL);
 		}
 	}
 
@@ -1539,9 +1546,13 @@ register CHARTYPE *text, *textend, *pat;
 			/* added by Udi 11/7/94 */
 			if(WORDBOUND) {
 				/* if(isalnum(*(unsigned char *)(text+1))) goto CONT; --> fixed by SHIOZAKI Takehiko <takehi-s@ascii.co.jp> */
-				if((text+1 <= textend) && isalnum(*(unsigned char *)(text+1)) && isalnum(*(unsigned char *)text)) goto CONT;	/* as if there was no match */
+				if((text+1 <= textend) && isalnum(*(unsigned char *)(text+1)) && isalnum(*(unsigned char *)text)) {
+					goto CONT;	/* as if there was no match */
+				}
 				/* if(isalnum(*(unsigned char *)(text-m))) goto CONT; --> fixed by SHIOZAKI Takehiko <takehi-s@ascii.co.jp> */
-				if((textbegin <= (text-m)) && isalnum(*(unsigned char *)(text-m)) && isalnum(*(unsigned char *)(text-m+1))) goto CONT;	/* as if there was no match */
+				if((textbegin <= (text-m)) && isalnum(*(unsigned char *)(text-m)) && isalnum(*(unsigned char *)(text-m+1))) {
+					goto CONT;	/* as if there was no match */
+				}
 				/* changed by Udi 11/7/94 to avoid having to set TR[] to W_delim */
 			}
 
@@ -1550,12 +1561,12 @@ register CHARTYPE *text, *textend, *pat;
 				if (!DELIMITER) {
 					curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 					if (*curtextbegin == '\n') curtextbegin ++;
-					curtextend = text+1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+					curtextend = curtextbegin /*text-m*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 					if (*curtextend == '\n') curtextend ++;
 				}
 				else {
 					curtextbegin = backward_delimiter(text, textbegin, tc_D_pattern, tc_D_length, OUTTAIL);
-					curtextend = forward_delimiter(text + 1, textend, tc_D_pattern, tc_D_length, OUTTAIL);
+					curtextend = forward_delimiter(curtextbegin /*text -m*/, textend, tc_D_pattern, tc_D_length, OUTTAIL);
 				}
 			}
 			else {
@@ -1563,12 +1574,12 @@ register CHARTYPE *text, *textend, *pat;
 				if (!DELIMITER) {
 					curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 					if (*curtextbegin == '\n') curtextbegin ++;
-					curtextend = text+1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+					curtextend = curtextbegin /*text-m*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 					if (*curtextend == '\n') curtextend ++;
 				}
 				else {
 					curtextbegin = backward_delimiter(text, textbegin, D_pattern, D_length, OUTTAIL);
-					curtextend = forward_delimiter(text + 1, textend, D_pattern, D_length, OUTTAIL);
+					curtextend = forward_delimiter(curtextbegin/*text -m*/, textend, D_pattern, D_length, OUTTAIL);
 				}
 			}
 
@@ -1857,24 +1868,24 @@ register CHARTYPE *text, *textend, *pat;
 					if (!DELIMITER) {
 						curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 						if (*curtextbegin == '\n') curtextbegin ++;
-						curtextend = text + 1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+						curtextend = curtextbegin /*text -m*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 						if (*curtextend == '\n') curtextend ++;
 					}
 					else {
 						curtextbegin = backward_delimiter(text, textbegin, tc_D_pattern, tc_D_length, OUTTAIL);
-						curtextend = forward_delimiter(text + 1, textend, tc_D_pattern, tc_D_length, OUTTAIL);
+						curtextend = forward_delimiter(curtextbegin /*text -m*/, textend, tc_D_pattern, tc_D_length, OUTTAIL);
 					}
 				}
 				else {
 					if (!DELIMITER) {
 						curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 						if (*curtextbegin == '\n') curtextbegin ++;
-						curtextend = text + 1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+						curtextend = curtextbegin/*text -m*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 						if (*curtextend == '\n') curtextend ++;
 					}
 					else {
 						curtextbegin = backward_delimiter(text, textbegin, D_pattern, D_length, OUTTAIL);
-						curtextend = forward_delimiter(text + 1, textend, D_pattern, D_length, OUTTAIL);
+						curtextend = forward_delimiter(curtextbegin/*text -m*/, textend, D_pattern, D_length, OUTTAIL);
 					}
 				}
 				textbegin = curtextend; /* (curtextend - 1 > textbegin ? curtextend - 1 : curtextend); */
@@ -2264,12 +2275,12 @@ register unsigned char *text, *pat, *textend;
 					if (!DELIMITER) {
 						curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 						if (*curtextbegin == '\n') curtextbegin ++;
-						curtextend = text + 1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+						curtextend = curtextbegin /*text -m*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 						if (*curtextend == '\n') curtextend ++;
 					}
 					else {
 						curtextbegin = backward_delimiter(text, textbegin, tc_D_pattern, tc_D_length, OUTTAIL);
-						curtextend = forward_delimiter(text + 1, textend, tc_D_pattern, tc_D_length, OUTTAIL);
+						curtextend = forward_delimiter(curtextbegin/*text -m*/, textend, tc_D_pattern, tc_D_length, OUTTAIL);
 					}
 				}
 				else {
@@ -2277,12 +2288,12 @@ register unsigned char *text, *pat, *textend;
 					if (!DELIMITER) {
 						curtextbegin = text; while((curtextbegin > textbegin) && (*(--curtextbegin) != '\n'));
 						if (*curtextbegin == '\n') curtextbegin ++;
-						curtextend = text + 1; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
+						curtextend = curtextbegin /*text -m*/; while((curtextend < textend) && (*curtextend != '\n')) curtextend ++;
 						if (*curtextend == '\n') curtextend ++;
 					}
 					else {
 						curtextbegin = backward_delimiter(text, textbegin, D_pattern, D_length, OUTTAIL);
-						curtextend = forward_delimiter(text + 1, textend, D_pattern, D_length, OUTTAIL);
+						curtextend = forward_delimiter(curtextbegin/*text -m*/, textend, D_pattern, D_length, OUTTAIL);
 					}
 				}
 				textbegin = curtextend; /*(curtextend - 1 > textbegin ? curtextend - 1 : curtextend); */
