@@ -568,7 +568,7 @@ int first_time;
 			if (i*8*sizeof(int) + j >= OneFilePerBlock) break;
 			index_tab[i] |= mask_int[j];
 		    }
-		    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH)) for (i=0; i<OneFilePerBlock; i++) {	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+		    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH)) for (i=0; i<OneFilePerBlock; i++) {	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 			free_list(&offsetptr[i]);
 			free_list(&offset_tab[i]);
 		    }
@@ -581,7 +581,7 @@ int first_time;
 		if (ByteLevelIndex && !NOBYTELEVEL && (RecordLevelIndex || !(Only_first && !PRINTAPPXFILEMATCH))) {
 		    for (i=0; i<OneFilePerBlock; i++) {
 			sorted_union(&offset_tab[i], &offsetptr[i], &index_tab[REAL_PARTITION - 2], dest_index_set[REAL_PARTITION - 2], 0);
-			if (!RecordLevelIndex && NOBYTELEVEL) {	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+			if (!RecordLevelIndex && NOBYTELEVEL) {	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 			    for (iii=0; iii<OneFilePerBlock; iii++) {
 				free_list(&offset_tab[iii]);
 				free_list(&offsetptr[iii]);
@@ -606,7 +606,7 @@ int first_time;
 			}
 		    }
 		    first_time = 0;
-		    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH)) for (i=0; i<OneFilePerBlock; i++) {	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+		    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH)) for (i=0; i<OneFilePerBlock; i++) {	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 			free_list(&offsetptr[i]);
 			free_list(&offset_tab[i]);
 		    }
@@ -629,7 +629,7 @@ int first_time;
 		    }
 		}
 		else if (ret) {
-		    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH)) for (i=0; i<OneFilePerBlock; i++) free_list(&offsetptr[i]);	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+		    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH)) for (i=0; i<OneFilePerBlock; i++) free_list(&offsetptr[i]);	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 		}
 		else {
 		    for (i=0; i<round(OneFilePerBlock, 8*sizeof(int)); i++) index_tab[i] &= dest_index_set[i];
@@ -638,7 +638,7 @@ int first_time;
 			    first_time = 0;
 			    for (i=0; i<OneFilePerBlock; i++) {
 				sorted_union(&offset_tab[i], &offsetptr[i], &index_tab[REAL_PARTITION - 2], dest_index_set[REAL_PARTITION - 2], 0);
-				if (!RecordLevelIndex && NOBYTELEVEL) {	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+				if (!RecordLevelIndex && NOBYTELEVEL) {	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 				    for (iii=0; iii<OneFilePerBlock; iii++) {
 					free_list(&offset_tab[iii]);
 					free_list(&offsetptr[iii]);
@@ -858,7 +858,7 @@ int first_time;
 				if (i*8*sizeof(int) + j >= OneFilePerBlock) break;
 				index_tab[i] |= mask_int[j];
 			    }
-			    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH))	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+			    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH))	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 			      for (i=0; i<OneFilePerBlock; i++) {
 			        for (patnum=0;patnum<num_mgrep_pat;patnum++)
 				  free_list(&multi_dest_offset_table[sorted[patnum]][i]);
@@ -874,7 +874,7 @@ int first_time;
 			    for (i=0; i<OneFilePerBlock; i++) {
 				sorted_union(&offset_tab[i], &multi_dest_offset_table[sorted[patnum]][i], &index_tab[REAL_PARTITION - 2], multi_dest_index_set[sorted[patnum]][REAL_PARTITION - 2], 0);
 
-				if (!RecordLevelIndex && NOBYTELEVEL) {	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+				if (!RecordLevelIndex && NOBYTELEVEL) {	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 				    for (iii=0; iii<OneFilePerBlock; iii++) {
 					for (jjj=0; jjj<num_mgrep_pat; jjj++)
 					    free_list(&multi_dest_offset_table[jjj][iii]);
@@ -900,7 +900,7 @@ int first_time;
 				}
 			    }
 			    first_time = 0;
-			    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH))	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+			    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH))	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 			      for (i=0; i<OneFilePerBlock; i++) {
 			        for (patnum=0;patnum<num_mgrep_pat;patnum++)
 			 	  free_list(&multi_dest_offset_table[sorted[patnum]][i]);
@@ -925,7 +925,7 @@ int first_time;
 			    }
 			}
 			else if (allindexmark[sorted[patnum]]) {
-			    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH))	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+			    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH))	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 			      for (i=0; i<OneFilePerBlock; i++) free_list(&multi_dest_offset_table[sorted[patnum]][i]);
 			}
 			else {
@@ -935,7 +935,7 @@ int first_time;
 				    first_time = 0;
 				    for (i=0; i<OneFilePerBlock; i++) {
 					sorted_union(&offset_tab[i], &multi_dest_offset_table[sorted[patnum]][i], &index_tab[REAL_PARTITION - 2], multi_dest_index_set[sorted[patnum]][REAL_PARTITION - 2], 0);
-					if (!RecordLevelIndex && NOBYTELEVEL) {	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+					if (!RecordLevelIndex && NOBYTELEVEL) {
 					    for (iii=0; iii<OneFilePerBlock; iii++) {
 						for (jjj=0; jjj<num_mgrep_pat; jjj++)
 						    free_list(&multi_dest_offset_table[jjj][iii]);
@@ -1301,9 +1301,9 @@ search_index(tree)
 					if (i*8*sizeof(int) + j >= OneFilePerBlock) break;
 					src_index_set[i] |= mask_int[j];
 				    }
-				    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH)) for (i=0; i<OneFilePerBlock; i++) {	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+				    if (ByteLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH)) for (i=0; i<OneFilePerBlock; i++) {
 					free_list(&curr_offset_table[i]);
-					free_list(&src_offset_table[i]);
+					if (!RecordLevelIndex) free_list(&src_offset_table[i]);	/* collect as many offsets as possible with RecordLevelIndex: free src_offset_table at the end of process_query() */
 				    }
 				    if (ByteLevelIndex && !RecordLevelIndex) NOBYTELEVEL = 1;
 				    goto quit;
@@ -1313,7 +1313,7 @@ search_index(tree)
 				if (ByteLevelIndex && !NOBYTELEVEL && (RecordLevelIndex || !(Only_first && !PRINTAPPXFILEMATCH))) {
 				    for (i=0; i<OneFilePerBlock; i++) {
 					sorted_union(&src_offset_table[i], &curr_offset_table[i], &src_index_set[REAL_PARTITION - 2], curr_index_set[REAL_PARTITION - 2], 0);
-					if (!RecordLevelIndex && NOBYTELEVEL) {	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+					if (!RecordLevelIndex && NOBYTELEVEL) {	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 					    for (iii=0; iii<OneFilePerBlock; iii++) {
 						free_list(&src_offset_table[iii]);
 						free_list(&curr_offset_table[iii]);
@@ -1338,9 +1338,9 @@ search_index(tree)
 					}
 				    }
 				    first_time = 0;
-				    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH)) for (i=0; i<OneFilePerBlock; i++) {	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+				    if (ByteLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH)) for (i=0; i<OneFilePerBlock; i++) {	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 					free_list(&curr_offset_table[i]);
-					free_list(&src_offset_table[i]);
+					if (!RecordLevelIndex) free_list(&src_offset_table[i]);	/* collect as many offsets as possible with RecordLevelIndex: free src_offset_table at the end of process_query() */
 				    }
 				    if (ByteLevelIndex && !RecordLevelIndex) NOBYTELEVEL = 1;
 				    /*
@@ -1360,7 +1360,7 @@ search_index(tree)
 				    }
 				}
 				else if (curr_index_set[REAL_PARTITION - 1] == 1) {
-				    if (ByteLevelIndex && !RecordLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH))	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+				    if (ByteLevelIndex && !NOBYTELEVEL && !(Only_first && !PRINTAPPXFILEMATCH))	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 				      for (i=0; i<OneFilePerBlock; i++) free_list(&curr_offset_table[i]);
 				}
 				else {
@@ -1370,7 +1370,7 @@ search_index(tree)
 					    first_time = 0;
 					    for (i=0; i<OneFilePerBlock; i++) {
 						sorted_union(&src_offset_table[i], &curr_offset_table[i], &src_index_set[REAL_PARTITION - 2], curr_index_set[REAL_PARTITION - 2], 0);
-						if (!RecordLevelIndex && NOBYTELEVEL) {	/* I don't know if !RecordLevelIndex && should be here: but only bad thing is that some offsets won't be freed until later... */
+						if (!RecordLevelIndex && NOBYTELEVEL) {	/* collect as many offsets as possible with RecordLevelIndex: free offset_tables at the end of process_query() */
 						    for (iii=0; iii<OneFilePerBlock; iii++) {
 							free_list(&src_offset_table[iii]);
 							free_list(&curr_offset_table[iii]);
