@@ -43,6 +43,7 @@ extern int FirstWordOfInfoIsKey;
 extern int UseFilters;
 extern int ByteLevelIndex;
 extern int RecordLevelIndex;
+extern int StoreByteOffset;
 extern char rdelim[MAX_LINE_LEN];
 extern char old_rdelim[MAX_LINE_LEN];
 extern int rdelim_len;
@@ -106,6 +107,7 @@ determine_sync()
 
 	strcpy(sync_path, "sync");
 	sprintf(S, "exec whereis sync > /tmp/zz.%d", getpid());
+	/* Change it to use which: not urgent. */
 	system(S);
 	sprintf(S, "/tmp/zz.%d", getpid());
 	if ((fp = fopen(S, "r")) == NULL) {
@@ -213,6 +215,10 @@ char **argv;
 	    ByteLevelIndex = ON;
 	    argc--; argv++;
 	}
+	else if(strcmp(argv[1], "-O") == 0) {
+	    StoreByteOffset = ON;
+	    argc--; argv++;
+	}
 	else if(strcmp(argv[1], "-r") == 0) {
 	    ByteLevelIndex = ON;
 	    RecordLevelIndex = ON;
@@ -409,6 +415,11 @@ char **argv;
 	return usage(1);
     }
 
+    if (StoreByteOffset && !RecordLevelIndex) {
+	fprintf(stderr, "ignoring -O since -r was not specified\n");
+	StoreByteOffset = OFF;
+    }
+
     if (InfoAfterFilename && !FilenamesOnStdin) {
 	fprintf(stderr, "-U works only when -F is specified!\n");
 	return usage(1);
@@ -1022,7 +1033,7 @@ docleanup:
 
     printf("\nIndex-directory: \"%s\"\nGlimpse-files created here:\n", INDEX_DIR);
     chdir(INDEX_DIR);
-    sprintf(s, "exec %s -lg .glimpse_* > /tmp/%d\n", SYSTEM_LS, pid);
+    sprintf(s, "exec %s -l .glimpse_* > /tmp/%d\n", SYSTEM_LS, pid);
     system(s);
     sprintf(s, "/tmp/%d", pid);
     if ((tmpfp = fopen(s, "r")) != NULL) {
@@ -1083,7 +1094,8 @@ int	flag;
 	fprintf(stderr, "-M X: use X MBytes of memory for temporary tables\n");
         fprintf(stderr, "-n [X]: index numbers as well as words; warn (into .glimpse_messages)\n\tif file adds > X%% numeric words: default is %d\n", DEF_NUMERIC_WORD_PERCENT);
 	fprintf(stderr, "-o: build a small (rather than tiny) size index (the recommended option!)\n");
-	fprintf(stderr, "-r deilm: build an index at the granulatiry of delimiter `delim'\n\tto do booleans by reading ONLY the index\n");
+	/*fprintf(stderr, "-O: when using -r option, store byte offset of each record,\n\tinstead of the record number, for faster access\n");*/
+	fprintf(stderr, "-r delim: build an index at the granularity of delimiter `delim'\n\tto do booleans by reading ONLY the index\n");
 	fprintf(stderr, "-R: recompute .glimpse_filenames_index from .glimpse_filenames if it changes\n");
 	fprintf(stderr, "-s: build index to support structured (Harvest SOIF type) queries\n");
 	fprintf(stderr, "-S X: adjust the size of the stop list\n");
